# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_benchmark.ipynb.

# %% auto 0
__all__ = ['benchmark']

# %% ../nbs/00_benchmark.ipynb 4
import time

from itertools import count
import torch
from torch import nn
from torch.nn import functional as F
from torch.cuda.amp.autocast_mode import autocast

import timm
from tqdm.auto import tqdm

# %% ../nbs/00_benchmark.ipynb 5
def benchmark(model: nn.Module, # Model to run
                bs: int =32,    # Batch size
                n_batches: int =None,  # Number of batches to run. `seconds` must be None
                n_seconds: int =None,  # Number of seconds to run. `n_batches` must be None
                fp16: int =False,      # Use Automatic Mixed Precision
                size: int=224,         # Mock-train on this size "images"
                dev: torch.device=torch.device("cuda:0"),): # Device to run on
    """Mock-train the model on random noise input."""

    # There can be only one
    assert not n_batches or not n_seconds
    assert n_batches or n_seconds


    torch.backends.cudnn.benchmark=True
    assert torch.backends.cudnn.is_available()

    model.to(dev)
    optim = torch.optim.SGD(model.parameters(), lr=0.00001, weight_decay=0.00005, momentum=0.1)

    X = torch.randn((bs, 3, size, size), device=dev)
    y = torch.randint(0, 999, (bs,), device=dev)

    if n_batches:
        pbar = tqdm(total=n_batches, unit="Batch")
    else:
        pbar = tqdm(total=n_seconds,
            bar_format="{l_bar}{bar}| {n:.1f}/{total} s [{elapsed}<{remaining} {postfix}]")

    start_time, last_time = 0, 0
    for c in count():
        with autocast(enabled=fp16):
            yhat = model(X)
            loss = F.cross_entropy(yhat, y)

        loss.backward()
        optim.step()
        optim.zero_grad(set_to_none=True)

        tt=time.time()

        if not last_time:
            pbar.write("Discarding the first iteration")

        if n_batches:
            if c == 0:
                last_time, start_time = tt, tt
            else:
                pbar.update()
                # Note: c starts with 0, but we discard the first iteration
                if c == n_batches:
                    break
        else:
            if not last_time:
                last_time, start_time = tt, tt
            else:
                iter_time =  tt - last_time
                run_time = tt - start_time
                pbar.update(iter_time)
                if run_time >= n_seconds:
                    break
                last_time = tt
    pbar.close()

    return ((time.time() - start_time), c*bs)




# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_cli.ipynb.

# %% auto 0
__all__ = ['cli']

# %% ../nbs/01_cli.ipynb 3
import platform

import torch
import timm
import wandb

from fastcore.script import *

from .benchmark import *

# %% ../nbs/01_cli.ipynb 4
@call_parse
def cli(wnb: str ="disabled",       # W&B mode. Accepted values: online, offline, disabled.
        wnb_run: str =None,    # W&B run name (auto-generate if None)
        wnb_project: str =None,
        wnb_entity: str =None,
        run_number = 1,             # A unique number to keep track over repeat runs
        model: str ="resnet50",     # TIMM Model name
        bs: int =32,                #Batch size
        size: int =224,             # (fake) image size
        fp16: bool =False,
        n_batches = 0,              # Run for N batches. Mututally exclusive with `n_seconds`
        n_seconds = 0,              # Run for N seconds. Mutually exclusive with `n_batches`
    ):

    if not n_batches and not n_seconds:
        print("Either `n_batches` or `n_seconds` must be non-zero")
        exit(1)
    if n_batches and n_seconds:
        print(f"`n_batches` and `n_seconds` are mutually explusive. {n_batches=}, {n_seconds=}")
        exit(1)

    assert wnb in ["online", "offline", "disabled"]
    
    stats = {
        "device_name" : torch.cuda.get_device_name(),
        "device_capability": str(torch.cuda.get_device_capability()),
        "pytorch": torch.version.__version__,
        "cuda": torch.version.cuda,
        "platform": platform.platform(),
        "model": model,
        "fp16": fp16,
        "bs": bs,
        "n_seconds": n_seconds if n_seconds else None,
        "n_batches": n_batches if n_batches else None,
        "run_number": run_number, 
        "gpu_mem" : torch.cuda.get_device_properties(0).total_memory,
    }

    print("\n".join([ f"{k}: {v}" for k, v in stats.items() ]))

    run =  wandb.init(mode=wnb, project=wnb_project, entity=wnb_entity, name=wnb_run)
    run.log(stats, step=0, commit=True)

    model = timm.create_model(model, pretrained=False)
    duration, n_items = benchmark(model, bs=bs, size=size, fp16=fp16, n_batches=n_batches, n_seconds=n_seconds)
    summary = {
        "duration": duration,
        "n_items": n_items,
        "throughput": n_items / duration,
    }

    print("\n".join([ f"{k}: {v}" for k, v in summary.items() ]))

    run.log(summary, step=1, commit=True)
    run.finish()






# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/ready-steady-go',
                'doc_host': 'https://xl0.github.io',
                'git_url': 'https://github.com/xl0/ready-steady-go',
                'lib_path': 'ready_steady_go'},
  'syms': { 'ready_steady_go.benchmark': { 'ready_steady_go.benchmark.benchmark': ( 'benchmark.html#benchmark',
                                                                                    'ready_steady_go/benchmark.py')},
            'ready_steady_go.cli': {'ready_steady_go.cli.cli': ('cli.html#cli', 'ready_steady_go/cli.py')}}}